/****** Object:  Stored Procedure dbo.sp_chg    Script Date: 1/24/98 7:05:41 PM ******/
SET QUOTED_IDENTIFIER ON
GO

if exists (select * from sysobjects where id = object_id('dbo.sp_chg') and sysstat & 0xf = 4)
	drop procedure "dbo"."sp_chg"
GO

/****** Object:  Stored Procedure dbo.sp_chg    Script Date: 11/28/97 2:28:14 AM ******/
/****** Object:  Stored Procedure dbo.sp_chg    Script Date: 10/17/97 2:58:00 PM ******/
CREATE PROCEDURE sp_chg 
	@guest_number varchar(8),
	@old_room_number varchar(8),
	@old_room_shareflag smallint,
	@new_room_number varchar(8),
	@new_room_shareflag smallint,
        @rebootflag smallint OUTPUT
        WITH RECOMPILE
AS
/****************
 * Declarations *
 ****************/
DECLARE @old_rn_status int

DECLARE @new_rn_status int
DECLARE @general_status smallint 
DECLARE @max_chargeflag smallint
DECLARE @next_chargeflag smallint
DECLARE @old_numeric    int
DECLARE @new_numeric    int
/* @general_status
 * 1 - RN(Y) RO(Y)
 * 2 - RN(Y) RO(N)
 * 3 - RN(N) RO(Y)
 * 4 - RN(N) RO(N)
 */
/************
 * Options  *
 ************/
SET NOCOUNT ON
/*
 * Name: 	sp_chg
 * Author:	Gavin
 * Function:	Process GC records from PMS.
 * Returns:
 *	1  - ok
 *	2  - one of @guest_number, [90030]
 *		    @old_room_number,
 *                  @old_room_shareflag, 
 *                  @new_room_number,
 *                  @new_room_shareflag is NULL
 *	3  - room_allocation does not exist
 *	4  - sci_xtended does not exist
 *	5  - nonsci_xtended does not exist
 *	6  - delete or update of room_allocation failed
 *      7  - cannot move someone who has not checkined
 *	8  - room_range does not exist
 *	9  - Both old and new rooms are outside of scope
 *      10 - Old and new room numbers cannot be the same
 *
 * Change history
 * --------------
 * Aug 13: Was born.
 * Nov 30: Changed interface to raiserror(90031) to take
 *         2 new params - @guest_number and @old_room_number.
 */
/******************* 
 * Data validation *
 *******************/
if @guest_number is NULL OR @old_room_number is NULL or
   @old_room_shareflag is NULL OR @new_room_number is NULL or
   @new_room_shareflag is NULL
BEGIN
	raiserror (90020,1,-1) WITH LOG
	return 2
END
if @old_room_number = @new_room_number
BEGIN
	raiserror (90034,1,-1) WITH LOG
	return 10
END
/********************************
 * Trap for non-existent tables *
 ********************************/
if not exists (select * from sysobjects 
              where id = object_id('dbo.room_allocation')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90001,1,-1) WITH LOG
	return 3
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.sci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90002,1,-1) WITH LOG
	return 4
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.nonsci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90003,1,-1) WITH LOG
	return 5
END
/**************************************************
 * Check whether @guest_number exists already in  *
 * room_allocation.                               *
 **************************************************/
if exists (select * from room_allocation 
           where guest_num = @guest_number)
BEGIN
/*
 * Guest already has record in room_allocation 
 */
   /* select 'Guest exists in room_allocation' */
   goto is_RO_RN_important
END
else
BEGIN
/*
 * Guest doesn't exists in room_allocation 
 */
   raiserror( 90031, 1, -1, @guest_number, @old_room_number ) WITH LOG
   return 7
END
/* FLOWCHART -> Is RO & RN within our ranges of interest? */
is_RO_RN_important:
	SELECT @old_numeric = CONVERT(INT, @old_room_number)
	SELECT @new_numeric = CONVERT(INT, @new_room_number)
        EXEC @old_rn_status = sp_isinrange @old_numeric
	EXEC @new_rn_status = sp_isinrange @new_numeric 
/* SELECT 'old_rn_status = ', @old_rn_status */
/* SELECT 'new_rn_status = ', @new_rn_status */
	if (@old_rn_status = 3 OR @new_rn_status = 3)
	BEGIN
		raiserror( 90032, 1, -1 ) WITH LOG
		return 8
	END
/* 1 - RN(Y) RO(Y)
 * 2 - RN(Y) RO(N)
 * 3 - RN(N) RO(Y)
 * 4 - RN(N) RO(N)
 */
	SELECT @general_status =
	CASE 
	WHEN @old_rn_status=1 AND @new_rn_status=1 THEN 1
	WHEN @old_rn_status=1 AND @new_rn_status=2 THEN 2
	WHEN @old_rn_status=2 AND @new_rn_status=1 THEN 3
	WHEN @old_rn_status=2 AND @new_rn_status=2 THEN 4
	ELSE 5
	END
/* SELECT 'general status is ', @general_status */
	if (@general_status = 5)
		goto decide_to_reboot
	if (@general_status = 4)
		goto falsefalse
	if (@general_status = 3)
		goto falsetrue
	if (@general_status = 2)
		goto truefalse
	if (@general_status = 1)
		goto truetrue
	
/* FLOWCHART -> Update room_allocation */	

falsefalse:
	raiserror( 90033, 1, -1 ) WITH LOG
	return 9
truefalse:
falsetrue:
truetrue:
	if exists(select * from room_allocation
                  where room_num=@new_room_number)
	BEGIN
	   /* somebody already staying there */
	   select @max_chargeflag = MAX(chargeflag) from
		  room_allocation where 
		  room_num=@new_room_number
	   select @next_chargeflag = @max_chargeflag + 1
	   	
 	   /* update room number and chargeflag */
	   update room_allocation SET
	   room_num=@new_room_number,
           chargeflag=@next_chargeflag
           where guest_num=@guest_number AND room_num=@old_room_number
	   if @@error <> 0
           BEGIN
		raiserror( 90004, 1, -1 ) WITH LOG
		return 6
           END
           else
	        goto decide_to_reboot
	END
	else
	BEGIN
	   /* empty room */
	   update room_allocation SET
	   room_num=@new_room_number,
           chargeflag=1
           where guest_num=@guest_number AND room_num=@old_room_number
	   if @@error <> 0
           BEGIN
		raiserror( 90004, 1, -1 ) WITH LOG
		return 6
           END
           else
	        goto decide_to_reboot
	END
/* FLOWCHART */
decide_to_reboot:

	
	/* select 'Check if needs to reboot:' */
	if exists (select * from room_allocation 
                   where room_num=@old_room_number)
	BEGIN
		/* Got people still in the room */
		/* select 'Reboot = No' */
		select @rebootflag = 0
	END
	else
	BEGIN
		/* Empty room */
		/* select 'Reboot = Yes' */
		select @rebootflag = 1
	END        
	
	return 1 /* All is well */

GO

SET QUOTED_IDENTIFIER OFF
GO
