/****** Object:  Stored Procedure dbo.sp_chg2    Script Date: 1/24/98 6:18:40 PM ******/
SET QUOTED_IDENTIFIER ON
GO
if exists (select * from sysobjects where id = object_id('dbo.sp_chg2') and sysstat & 0xf = 4)
	drop procedure "dbo"."sp_chg2"
GO

CREATE PROCEDURE sp_chg2
	@guest_number varchar(8),
	@room_number varchar(8),
	@sci_number varchar(16) = '0',
	@salutation varchar(32) = 'Honorable',
	@lastname varchar(255)  = 'Guest',
	@firstname varchar(255) = 'Sheraton',
	@shareflag smallint,     
	@vipflag varchar(4)     = 'No',
	@fideliolang varchar(2) = 'EA'
AS
/****************
 * Declarations *
 ****************/

/************
 * Options  *
 ************/
SET NOCOUNT ON

/*
 * Name: 	sp_chg2
 * Author:	Gavin/Phang

 * Function:	Process GC records type 2 from PMS.
 * Note:	We will not use @shareflag for any logic.
 * Returns:
 *	1  - ok
 *	2  - @guest_number is NULL
 *	3  - room_allocation does not exist
 *	4  - sci_xtended does not exist
 *	5  - nonsci_xtended does not exist
 *      6  - Guest with G# @guest_number non existent
 *      7  - Cannot update room_allocation
 *
 * Change history
 * --------------
 * Nov 30: Was born.        
 */




/******************* 
 * Initialization  *
 *******************/

/******************* 
 * Data validation *
 *******************/
if @guest_number is NULL
BEGIN
	raiserror (90040,1,-1) WITH LOG 
	return 2
END

/********************************
 * Trap for non-existent tables *
 ********************************/
if not exists (select * from sysobjects 
              where id = object_id('dbo.room_allocation')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90001,1,-1) WITH LOG 
	return 3
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.sci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90002,1,-1) WITH LOG 
	return 4
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.nonsci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90003,1,-1) WITH LOG 
	return 5
END



if exists (select * from room_allocation where
           guest_num = @guest_number)
BEGIN
	update room_allocation 
	set room_num=@room_number,
	    sci_num=@sci_number,
	    salutation=@salutation,
	    lastname=@lastname,
	    firstname=@firstname,
	    shareflag=@shareflag,
	    vipflag=@vipflag,
	    fideliolang=@fideliolang
        where guest_num=@guest_number
	if @@error <> 0
	BEGIN
		raiserror (90042,1,-1,@guest_number,@room_number)
		return 7
	END
END
else
BEGIN
	raiserror (90041,1,-1,@guest_number)
	return 6
END


die:
	return 1

GO

SET QUOTED_IDENTIFIER OFF
GO
