/****** Object:  Stored Procedure dbo.sp_del    Script Date: 1/24/98 6:33:55 PM ******/
SET QUOTED_IDENTIFIER ON
GO
if exists (select * from sysobjects where id = object_id('dbo.sp_del') and sysstat & 0xf = 4)
	drop procedure "dbo"."sp_del"
GO

CREATE PROCEDURE sp_del 
	@guest_number varchar(8),
	@room_number varchar(8),
	@swapflag smallint = 0,      -- 0/1
        @rebootflag smallint OUTPUT
        WITH RECOMPILE
AS
/****************
 * Declarations *
 ****************/
DECLARE @sci_number varchar(16)
DECLARE @gotswapflag int


/************
 * Options  *
 ************/
SET NOCOUNT ON
/*
 * Name: 	sp_del
 * Author:	Gavin/Phang
 * Function:	Process GO records from PMS.
 * Returns:
 *	1  - ok
 *	2  - @room_number is NULL

 *	3  - room_allocation does not exist
 *	4  - sci_xtended does not exist
 *	5  - nonsci_xtended does not exist
 *	6  - delete or update of room_allocation failed
 *	7  - delete or update of nonsci_xtended failed
 *	8  - cannot checkout when you're not checkined.
 *	9  - @guest_number is NULL and @swapflag = 0
 *
 * Change history
 * --------------
 * Aug 3: Was born.
 * Aug 4: Added @rebootflag to convey to the calling program
 *	  whether it needs to reboot the STB or not.
 * Aug 12 : Remove output message - SELECT 'messsage'
 * Oct 21 : Changed line near "Check whether @guest_number exists 
 *          already in room_allocation" from 
 *		guest_num -> room_num
 *		guest_number -> room_number
 * Oct 23 : Changed line near "Check whether @guest_number exists 
 *          already in room_allocation" from 
 *		room_num -> guest_num
 *		room_number -> guest_number
 * Nov 27 : Changed system message 90021 and how it is
 *          called by adding @guest_number & @room_number
 *          as input.
 * Nov 30 : Changed message 90020 such that it doesn't 
 *          include G#
 */
/******************* 
 * Data validation *
 *******************/
if @room_number is NULL
BEGIN
	raiserror (90020,1,-1) WITH LOG
	return 2
END


if ((@guest_number is NULL) AND (@swapflag = 0))
BEGIN
	raiserror (90024,1,-1,@room_number) WITH LOG
	return 9
END


if ((@guest_number is NULL) AND (@swapflag = 1))
BEGIN
	select @gotswapflag = 1
END
else
BEGIN
	select @gotswapflag = 0
END



/********************************
 * Trap for non-existent tables *

 ********************************/
if not exists (select * from sysobjects 
              where id = object_id('dbo.room_allocation')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90001,1,-1) WITH LOG
	return 3
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.sci_xtended')
              AND sysstat & 0xf = 3)
BEGIN

	raiserror (90002,1,-1) WITH LOG
	return 4
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.nonsci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90003,1,-1) WITH LOG
	return 5
END

if @gotswapflag <> 0
BEGIN
	if exists (select * from room_allocation where
                   room_num = @room_number)
	BEGIN
		delete from room_allocation where room_num=@room_number
		goto swapffg
	END
	else
	BEGIN
		raiserror (90025,1,-1,@room_number)
		return 8
	END
END
else
BEGIN
/**************************************************
 * Check whether @guest_number exists already in  *
 * room_allocation at room @room_number !         *
 **************************************************/
if exists (select * from room_allocation 
           where guest_num = @guest_number AND
                 room_num = @room_number)
BEGIN
/*
 * Guest already has record in room_allocation 
 */
   /* select 'Guest exists in room_allocation' */
   goto is_this_n_SCI_member
END
else
BEGIN
/*
 * Guest doesn't exists in room_allocation 
 */
   raiserror( 90021, 1, -1, @guest_number, @room_number ) WITH LOG
   return 8
END

END -- @gotswapflag





/* FLOWCHART */
is_this_n_SCI_member:
	   select @sci_number = NULL
	/* Check if user is an SCI member */
	   select @sci_number=sci_num from room_allocation
                  where guest_num=@guest_number AND
                        room_num=@room_number


	   if (@sci_number is NOT NULL 
               and @sci_number <> '0')
           BEGIN
		/* select 'SCI' */
		delete from room_allocation where
                  guest_num=@guest_number AND
                  room_num=@room_number
                if @@error <> 0
                BEGIN

			raiserror(90022,1,-1) WITH LOG
			return 6
                END
		goto decide_to_reboot
           END
           else
	   BEGIN
		/* select 'NONSCI' */
		delete from room_allocation where
                  guest_num=@guest_number AND
                  room_num=@room_number
		
		if @@error <> 0
                BEGIN
			raiserror(90022,1,-1) WITH LOG
			return 6
                END
		
		delete from nonsci_xtended where
                  guest_num=@guest_number
		if @@error <> 0
                BEGIN
			raiserror(90023,1,-1) WITH LOG

			return 7
                END
		goto decide_to_reboot
	   END

/* FLOWCHART */
decide_to_reboot:
	
	/* select 'Check if needs to reboot:' */
	if exists (select * from room_allocation 
                   where room_num=@room_number)
	BEGIN


		/* Got people still in the room */
		/* select 'Reboot = No' */
		select @rebootflag = 0

	END

	else
	BEGIN
		/* Empty room */
		/* select 'Reboot = Yes' */
		select @rebootflag = 1
	END        






swapffg:
	
	return 1 /* All is well */

GO

SET QUOTED_IDENTIFIER OFF
GO
