/****** Object:  Stored Procedure dbo.sp_ins    Script Date: 1/25/98 1:07:39 AM ******/
SET QUOTED_IDENTIFIER ON
GO
if exists (select * from sysobjects where id = object_id('dbo.sp_ins') and sysstat & 0xf = 4)
	drop procedure "dbo"."sp_ins"
GO

CREATE PROCEDURE sp_ins
	@guest_number varchar(8),
	@room_number varchar(8),
	@sci_number varchar(16) = '0',
	@salutation varchar(32) = 'Honorable',
	@lastname varchar(255)  = 'Guest',
	@firstname varchar(255),
	@shareflag smallint,     
	@swapflag smallint,
	@vipflag varchar(4)     = 'No',
	@fideliolang varchar(2) = 'EA'
AS
/****************
 * Declarations *
 ****************/
DECLARE @max_chargeflag  smallint
DECLARE @next_chargeflag smallint
DECLARE @is_checkin      smallint
DECLARE @is_sci          smallint
DECLARE @old_room_number varchar(8)
/************
 * Options  *
 ************/
SET NOCOUNT ON
/*
 * Name: 	sp_ins
 * Author:	Gavin/Phang
 * Function:	Process GI records from PMS.
 * Note:	We will not use @shareflag for any logic.
 * Returns:
 *	1  - ok
 *	2  - one of @guest_number,
 *		   @room_number,
 *		   @firstname,
 *		   @swapflag is NULL
 *	3  - room_allocation does not exist
 *	4  - sci_xtended does not exist
 *	5  - nonsci_xtended does not exist
 *	6  - insert into or update of room_allocation failed
 *	7  - insert into or update of sci_xtended failed
 *	8  - insert into or update of nonsci_xtended failed
 *	9  - Data validation: @swapflag out of range
 *	10 - A guest cannot checkin multiple times
 *	11 - Cannot update info for a guest which is
 *           not checkined.
 *
 *
 * Change history
 * --------------
 * Year 1997:
 * ==========
 * Aug 03: Change interface to receive 'swapflag'
 *         as a paramater.
 * Aug 12: Remove output message - SELECT 'messsage'
 * Aug 13: Modified update_room_alloc portion to decide
 *         between insert/update. We introduce:
 *	   a) new variable - @is_checkin
 *         b) new variable - @is_sci
 *         c) new variable - @old_room_number
 * Nov 30: Modified interface to raiserror(90008) to 
 *         take 2 params - RN and G#. This is to provide
 *         more user friendly error messages.
 * Nov 30: Added "return 1" to the "stop" marker.
 * Dec 23: Changed all inserts to support new field called
 *         "lastchanged" in both sci_xtended and room_allocation.
 *
 * Year 1998
 * =========
 * Jan 24: Updated to support new sci_xtended with userid/passwd fields.
 *        
 */




/******************* 
 * Initialization  *
 *******************/
select @is_checkin = 0 /* not checkin default */
select @is_sci     = 0 /* not sci member by default */
/******************* 
 * Data validation *
 *******************/
if @guest_number is NULL OR @room_number is NULL OR
   @firstname is NULL OR @swapflag is NULL
BEGIN

	raiserror (90000,1,-1) WITH LOG 
	return 2
END
if (@swapflag <> 1 AND @swapflag <> 0)
BEGIN
        raiserror (90007,1,-1) WITH LOG 
	return 9
END 
/********************************
 * Trap for non-existent tables *
 ********************************/
if not exists (select * from sysobjects 
              where id = object_id('dbo.room_allocation')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90001,1,-1) WITH LOG 
	return 3

END
if not exists (select * from sysobjects 
              where id = object_id('dbo.sci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90002,1,-1) WITH LOG 
	return 4
END
if not exists (select * from sysobjects 
              where id = object_id('dbo.nonsci_xtended')
              AND sysstat & 0xf = 3)
BEGIN
	raiserror (90003,1,-1) WITH LOG 
	return 5
END
/**************************************************
 * Check whether @guest_number exists already in  *
 * room_allocation.                               *
 **************************************************/
if exists (select * from room_allocation 
           where guest_num = @guest_number)
BEGIN
/*
 * Guest already has record in room_allocation 
 */
/*GAVIN*/
   /* select 'Guest exists in room_allocation' */
   select @is_checkin = 1
   if (@swapflag = 0)
   BEGIN
	/* SF does not exists */
        raiserror (90008,1,-1,@guest_number,@room_number) WITH LOG 
	return 10
   END
   
   if (@swapflag = 1)
   BEGIN
	/* SF exists */
	GOTO diagram2
   END
END
else
BEGIN
/*
 * Guest does not exist in room_allocation 
 */
/* GAVIN */
   /* select 'Guest does not exist in room_allocation' */
   select @is_checkin = 0
   if (@swapflag = 0)
   BEGIN
	/* SF does not exists */
	GOTO diagram2
   END
   
   if (@swapflag = 1)
   BEGIN
	/* SF exists */
	raiserror (90009,1,-1) WITH LOG 
	return 11
   END 
END
goto stop
/* FLOWCHART -> Continue: diagram 2 */
diagram2:
	/* Check if user is an SCI member */
	if @sci_number <> '0' AND @sci_number is NOT NULL
	BEGIN
/* USER IS SCI */
           select @is_sci = 1
	   if exists (select * from sci_xtended 
                      where sci_num = @sci_number)
           BEGIN
	      /* FLOWCHART -> Update sci_xtended (A0) */
	      update sci_xtended
                     set lastname=@lastname,
                         firstname=@firstname,	      
		                 salutation=@salutation
                     where sci_num=@sci_number

	      if @@error <> 0
              BEGIN
	      	raiserror(90005,1,-1) WITH LOG 
		return 7
	      END
	      else
                goto update_room_alloc
           END
           else
           BEGIN

	      /* FLOWCHART -> Insert a record (A0) 
               *              into sci_xtended
               */
	      insert into sci_xtended VALUES(
                     @sci_number,
		             @lastname,
                     @firstname,
                     @salutation,
                     DEFAULT,
					 DEFAULT,
					 NULL,
					 NULL)
	      if @@error <> 0
              BEGIN
	      	raiserror(90005,1,-1) WITH LOG 
		return 7
	      END
	      else
                goto update_room_alloc
           END     
	END
	else
	BEGIN
/* USER IS NOT SCI */
           select @is_sci = 0
	   if exists (select * from nonsci_xtended 
                      where guest_num = @guest_number)
           BEGIN
	      /* FLOWCHART -> Update G# record */
	      update nonsci_xtended

                     set lang=@fideliolang
                     where guest_num=@guest_number
	      if @@error <> 0
              BEGIN
	      	raiserror(90006,1,-1) WITH LOG 
		return 8
	      END
	      else
                goto update_room_alloc
           END
           else
           BEGIN
	      /* FLOWCHART -> Insert a record (G#) 
               *              into nonsci_xtended
               */
	      insert into nonsci_xtended VALUES(
                     @guest_number,

                     @fideliolang)
	      if @@error <> 0
              BEGIN	
         	raiserror(90006,1,-1) WITH LOG 
		return 8
	      END
	      else
                goto update_room_alloc
           END     
	END
/* FLOWCHART -> Update room_allocation (G#) */
update_room_alloc:
	/* FLOWCHART: Is this an occupied room ? (direct query or GS) */
	if exists (select * from room_allocation where room_num = @room_number)
	BEGIN
	   /* This is a sharer room */
/* GAVIN */
           /* select 'This is a sharer room' */
	   goto occupied_room_checkin
	END
	else
	BEGIN
	   /* This is an empty room */
/* GAVIN */
           /* select 'This is an empty room' */
	   goto empty_room_checkin
	END	
	goto stop
/* FLOWCHART -> Occupied Room Checkin Processing */
occupied_room_checkin:
	if (@swapflag = 0)
        BEGIN
        	if (@is_checkin = 0)
                BEGIN
	              select @max_chargeflag=MAX(chargeflag) from
                         room_allocation where room_num = @room_number
                  select @next_chargeflag = @max_chargeflag + 1
		  /* update */
/*GAVIN: Select 'SF=0, not checkined' */
		  /* insert */
	              insert into room_allocation VALUES(
                      @guest_number,
                      @room_number,
                      @sci_number,
                      @salutation,
                      @lastname,
                      @firstname,
                      @shareflag,
                      @vipflag,
                      @fideliolang,
                      @next_chargeflag,
					  DEFAULT)
	          if @@error <> 0
                  BEGIN
		              raiserror(90004,1,-1) WITH LOG 
		              return 6
	              END
              else
		          goto stop
                END
		else

                BEGIN
/*GAVIN: Select 'SF=0, already checkined' */
		   raiserror(90008,1,-1,@guest_number,@room_number) WITH LOG
		   return 10
                END
        END
	else
        BEGIN
/*====*/
        	if (@is_checkin = 0)
                BEGIN
	          select @max_chargeflag=MAX(chargeflag) from
                     room_allocation where room_num = @room_number
              select @next_chargeflag = @max_chargeflag + 1
		  /* insert */
	          insert into room_allocation VALUES(
                      @guest_number,
                      @room_number,
                      @sci_number,
                      @salutation,
                      @lastname,
                      @firstname,
                      @shareflag,
                      @vipflag,
                      @fideliolang,
                      @next_chargeflag,
					  DEFAULT)
	          if @@error <> 0
                  BEGIN
		    raiserror(90004,1,-1) WITH LOG 
		    return 6
	          END
                  else

		    goto stop
                END
		else
                BEGIN
		  /* --Is GI's RN equal to existing RN -- */
		  /* GI's RN = @room_number */
                  /* existing RN = select room_num from room_allocation where guest_num=@guest_number */
		  select @old_room_number=room_num from room_allocation where guest_num=@guest_number  
		  if (@old_room_number != @room_number)
                  BEGIN
		         select @max_chargeflag=MAX(chargeflag) from
                        room_allocation where room_num = @room_number
                 select @next_chargeflag = @max_chargeflag + 1
		     /* update */
	             update room_allocation SET
                      room_num=@room_number,
	                  sci_num=@sci_number,
                      salutation=@salutation,
                      lastname=@lastname,
                      firstname=@firstname,
                      shareflag=@shareflag,
                      vipflag=@vipflag,
                      fideliolang=@fideliolang,
                      chargeflag=@next_chargeflag
		         where guest_num=@guest_number
	             if @@error <> 0
                     BEGIN
		       raiserror(90004,1,-1) WITH LOG 
		       return 6
	             END
                     else
		       goto stop
                  END
                  else
                  BEGIN
		     /* update without affecting chargeflag */
	             update room_allocation SET
	              sci_num=@sci_number,
                      salutation=@salutation,
                      lastname=@lastname,
                      firstname=@firstname,
                      shareflag=@shareflag,
                      vipflag=@vipflag,
                      fideliolang=@fideliolang
		     where guest_num=@guest_number
	             if @@error <> 0
                     BEGIN
		       raiserror(90004,1,-1) WITH LOG 
		       return 6
	             END
                     else
		       goto stop
                  END
                END
        END
	
        goto stop
/* FLOWCHART -> Empty Room Checkin Processing */
empty_room_checkin:
	if (@swapflag = 0)
	   BEGIN
	      /* Insert into room_allocation with default chargeflag */
	      insert into room_allocation VALUES(
                      @guest_number,
                      @room_number,
                      @sci_number,
                      @salutation,
                      @lastname,
                      @firstname,
                      @shareflag,
                      @vipflag,
                      @fideliolang,
                      DEFAULT,
					  DEFAULT)
	      if @@error <> 0

              BEGIN
		raiserror(90004,1,-1) WITH LOG 
		return 6
	      END
              else
		goto stop		
           END
	else
	   BEGIN
	      /* Insert into room_allocation with default chargeflag */
	      insert into room_allocation VALUES(
                      @guest_number,
                      @room_number,
                      @sci_number,
                      @salutation,
                      @lastname,
                      @firstname,
                      @shareflag,
                      @vipflag,
                      @fideliolang,
                      DEFAULT,
					  DEFAULT)
	      if @@error <> 0
              BEGIN
		raiserror(90004,1,-1) WITH LOG 
		return 6
	      END

              else
		goto stop		
           END

	   goto stop
stop:
        return 1



GO

SET QUOTED_IDENTIFIER OFF
GO
